#include <stdint.h>
#include <stdbool.h>
#include "ble.h"
#include "ble_srv_common.h"

#define TAG_UUID_BASE {0x23, 0xD1, 0xBC, 0xEA, 0x5F, 0x78, 0x23, 0x15, 0xDE, 0xEF, 0x12, 0x12, 0x00, 0x00, 0x00, 0x00}
#define TAG_UUID_SERVICE     0x1523
#define TAG_UUID_LED_CHAR    0x1525
#define TAG_UUID_BUTTON_CHAR 0x1524

// Forward declaration of the ble_tag_t type. 
typedef struct ble_tag_s     ble_tag_t;

typedef void (*ble_tag_led_write_handler_t) (ble_tag_t *p_tag, uint8_t new_state);

typedef struct
{
    ble_tag_led_write_handler_t  led_write_handler;  //Event handler to be called when LED characteristic is written
	
} ble_tag_init_t;


//Our Custom Service structure. This contains various status details for the service
typedef struct ble_tag_s
{
    uint16_t                       service_handle;
    ble_gatts_char_handles_t       led_char_handles;
    ble_gatts_char_handles_t       button_char_handles;
    uint8_t                        uuid_type;
    uint16_t                       conn_handle;
    ble_tag_led_write_handler_t    led_write_handler;
	
} ble_tag_t;


//-----------------Function Prototypes----------------------------

/* Function for initializing the Custom TAG Service
 *  PARAMS 
 *   p_tag      - a pointer to a ble_tag_t structure that contains the service details (p_tag)
 *   p_tag_init - a pointer to the service initialization function which will modify the contents
 *                of the p_tag struct during intitialization
 *
 * RETURNS
 *   Returns NRF_SUCCESS on successful initialization of service, otherwise an error code.
 */
uint32_t ble_tag_init(ble_tag_t *p_tag, const ble_tag_init_t *p_tag_init);




/* Function for handling the Application's BLE Stack events.
 * Handles all events from the BLE stack of interest to our custom Service
 *  PARAMS
 *    p_tag      - a pointer to a ble_tag_t structure that contains the service details (p_tag)
 *    p_ble_evt  - a pointer to the event details structure received from the BLE stack
 */
void ble_tag_on_ble_evt(ble_tag_t *p_tag, ble_evt_t *p_ble_evt);




/* Function for sending button state notification.
 *  PARAMS
 *    p_tag         - a pointer to a ble_tag_t structure that contains the service details (p_tag)
 *    button_state  - The state of the button I/O pin
 */
uint32_t ble_tag_on_button_change(ble_tag_t *p_tag, uint8_t button_state);

